/* 
 * Copyright (c) 2010 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 * http://hardware.ispras.ru
 *
 * All rights reserved.
 *
 * This source code is free software; you can redistribute it
 * and/or modify it in source code form under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 * $Id: design.c,v 1.2 2010/03/04 15:40:44 kamkin Exp $
 */

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include "design.h"
#include "error.h"

///////////////////////////////////////////////////////////////////////////////
// Wire
///////////////////////////////////////////////////////////////////////////////

void vt_init_wire(vt_wire_t *wire, const char *name, vt_direct_t direct, vt_input_t input, int min, int max)
{
    assert(wire != NULL);
    assert(name != NULL);

    wire->name = name;
    wire->direct = direct;
    wire->input = input;
    wire->min = min;
    wire->max = max;
    wire->grouping_counter = 0;
}

vt_wire_t* vt_create_wire(const char *name, vt_direct_t direct, vt_input_t input, int min, int max)
{
    vt_wire_t *wire = (vt_wire_t*)malloc(sizeof(vt_wire_t));

    vt_init_wire(wire, name, direct, input, min, max);

    return wire;
}

void vt_destroy_wire(vt_wire_t *wire)
{
    if(!wire) { return; }

    free(wire);
}

///////////////////////////////////////////////////////////////////////////////
// Item
///////////////////////////////////////////////////////////////////////////////
void vt_init_item(vt_item_t *item, vt_wire_t *wire)
{
    assert(item != NULL);
    assert(wire != NULL);
    
    item->wire = wire;
    item->next = NULL;
}

vt_item_t* vt_create_item(vt_wire_t *wire)
{
    vt_item_t *item = (vt_item_t*)malloc(sizeof(vt_item_t));
    
    vt_init_item(item, wire);
    
    return item;
    
}

void vt_destroy_item(vt_item_t *item)
{
    if(!item) { return; }
	    
    if(!item->wire)
        { vt_destroy_wire(item->wire); }

    free(item);
}

///////////////////////////////////////////////////////////////////////////////
// List
///////////////////////////////////////////////////////////////////////////////
void vt_init_list(vt_list_t *list)
{
    assert(list != NULL);
	
    list->size = 0;
    list->head = NULL;
    list->last = NULL;
}

vt_list_t *vt_create_list(void)
{
    vt_list_t *list = (vt_list_t*)malloc(sizeof(vt_list_t));

    vt_init_list(list);
    
    return list;
}

void vt_add_list(vt_list_t *list, vt_wire_t *wire)
{
    assert(list != NULL);
    assert(wire != NULL);

    list->size++;

    if(!list->head)
        { list->last = list->head = vt_create_item(wire); }
    else
        { list->last = list->last->next = vt_create_item(wire); }
}

void vt_destroy_list(vt_list_t *list)
{
    vt_item_t *head, *next;

    if(!list) { return; }

    for(head = list->head; head; head = next)
    {
    	next = head->next;
    	vt_destroy_item(head);
    }

    free(list);
}

///////////////////////////////////////////////////////////////////////////////
// Design
///////////////////////////////////////////////////////////////////////////////

void vt_init_design(vt_design_t *design, const char *name)
{
    assert(design != NULL);
    assert(name != NULL);

    design->name = name;
    design->inputs = vt_create_list();
    design->outputs = vt_create_list();	
}

vt_design_t* vt_create_design(const char *name)
{
    vt_design_t *design = (vt_design_t*)malloc(sizeof(vt_design_t));

    vt_init_design(design, name);

    return design;
}

void vt_destroy_design(vt_design_t *design)
{
    if(!design) { return; }

    vt_destroy_list(design->inputs);
    vt_destroy_list(design->outputs);

    free(design);
}

void vt_add_input_design(vt_design_t *design, vt_wire_t *wire)
{
    assert(design != NULL);

    vt_add_list(design->inputs, wire);
}

void vt_add_output_design(vt_design_t *design, vt_wire_t *wire)
{
    assert(design != NULL);

    vt_add_list(design->outputs, wire);
}

int vt_find_and_update_similar_wire_in_design(vt_list_t *list, vt_wire_t *wire)
{
    assert(list != NULL);
    assert(wire != NULL);

    vt_item_t *item = list->head;
    vt_wire_t *lwire;
    while(item)
    {
        lwire = item->wire;
        if(strlen(lwire->name) == strlen(wire->name))
          if(!strncmp(lwire->name, wire->name, strlen(lwire->name) - 1) && lwire->min == wire->min && lwire->max == wire->max) {
            if(lwire->name[strlen(lwire->name) - 1] == '0' &&
                  wire->name[strlen(wire->name) - 1] - '0' == lwire->grouping_counter + 1)
            {
              lwire->grouping_counter++;
              return 1;
            }
          }
        item = item->next;
    }
    return 0;
}

int vt_find_and_update_similar_input_wire_in_design(vt_design_t *design, vt_wire_t *wire)
{
    assert(design != NULL);
    return vt_find_and_update_similar_wire_in_design(design->inputs, wire);
}

int vt_find_and_update_similar_output_wire_in_design(vt_design_t *design, vt_wire_t *wire)
{
    assert(design != NULL);
    return vt_find_and_update_similar_wire_in_design(design->outputs, wire);
}

void vt_reduce_wire_names(vt_list_t *list)
{
    vt_item_t *item = list->head;
    vt_wire_t *lwire;
    while(item)
    {
        lwire = item->wire;
        if(lwire->grouping_counter > 0)
        {
          char *tmp = (char*) lwire->name;
          tmp[strlen(tmp) > 0 ? strlen(tmp) - 1 : 0] = '\0';
        }
        item = item->next;
    }
}

void vt_reduce_grouped_wire_names(vt_design_t *design)
{
    assert(design != NULL);
    assert(design->inputs != NULL);
    assert(design->outputs != NULL);
    vt_reduce_wire_names(design->inputs);
    vt_reduce_wire_names(design->outputs);
}

