/* 
 * Copyright (c) 2010 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 * http://hardware.ispras.ru
 *
 * All rights reserved.
 *
 * This source code is free software; you can redistribute it
 * and/or modify it in source code form under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 * $Id: options.c,v 1.3 2010/03/04 15:40:44 kamkin Exp $
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "options.h"

vt_options_t options =
{
    .outputs     = ALL_OUTPUTS,
    .application = APPLICATION_DEFAULT,
    .clock_input = CLOCK_INPUT_DEFAULT,
    .reset_input = RESET_INPUT_DEFAULT,
    .rst_poswork = RST_POSWORK_DEFAULT,
    .cmode       = CMODE_DEFAULT,
    .module_name = MODULE_NAME_DEFAULT,
    .all_outputs = ALL_OUTPUTS_DEFAULT,
    .testbench_v = TESTBENCH_V_DEFAULT,
    .vpi_media_c = VPI_MEDIA_CPP_DEFAULT,
    .vpi_media_h = VPI_MEDIA_H_DEFAULT,
    .vpi_systf_c = VPI_SYSTF_CPP_DEFAULT,
    .vpi_systf_h = VPI_SYSTF_H_DEFAULT,
    .interface_c = INTERFACE_CPP_DEFAULT,
    .interface_h = INTERFACE_H_DEFAULT,
    .destination = DESTINATION_DEFAULT,
    .group       = GROUP_DEFAULT
};

int vt_defined_option(const char *value)
{
    return value != NULL && *value;
}

int vt_defined_and_empty_option(const char *value)
{
    return value != NULL && !strcmp(value, EMPTY_OPTION_VALUE);
}

int vt_defined_and_not_empty_option(const char *value)
{
    return value != NULL && *value && strcmp(value, EMPTY_OPTION_VALUE);
}

static char * vt_replace_ext(char *result, const char *file, const char *ext)
{
    char *p;

    strcpy(result, file);

    if(!(p = strrchr(result, '.')))
        { p = result + strlen(file); }

    strcpy(p, ext);

    return result;
}

static char *vt_get_vpi_media_h(const char *vpi_media_c)
{
    static char vpi_media_h[256];
    return vt_replace_ext(vpi_media_h, vpi_media_c, ".h");
}

static char *vt_get_vpi_systf_h(const char *vpi_systf_c)
{
    static char vpi_systf_h[256];
    return vt_replace_ext(vpi_systf_h, vpi_systf_c, ".h");
}

static char *vt_get_interface_h(const char *interface_c)
{
    static char interface_h[256];
    return vt_replace_ext(interface_h, interface_c, ".h");
}

int vt_process_options(ivl_design_t des, vt_options_t *options)
{
    const char *application = ivl_design_flag(des, APPLICATION_OPTION);
    const char *clock_input = ivl_design_flag(des, CLOCK_INPUT_OPTION);
    const char *reset_input = ivl_design_flag(des, RESET_INPUT_OPTION);
    const char *rst_poswork = ivl_design_flag(des, RST_POSWORK_OPTION);
    const char *cmode       = ivl_design_flag(des, CMODE_OPTION);
    const char *module_name = ivl_design_flag(des, MODULE_NAME_OPTION);
    const char *all_outputs = ivl_design_flag(des, ALL_OUTPUTS_OPTION);
    const char *testbench_v = ivl_design_flag(des, TESTBENCH_V_OPTION);
    const char *vpi_media_c = ivl_design_flag(des, VPI_MEDIA_C_OPTION);
    const char *vpi_media_h = ivl_design_flag(des, VPI_MEDIA_H_OPTION);
    const char *vpi_systf_c = ivl_design_flag(des, VPI_SYSTF_C_OPTION);
    const char *vpi_systf_h = ivl_design_flag(des, VPI_SYSTF_H_OPTION);
    const char *interface_c = ivl_design_flag(des, INTERFACE_C_OPTION);
    const char *interface_h = ivl_design_flag(des, INTERFACE_H_OPTION);
    const char *destination = ivl_design_flag(des, DESTINATION_OPTION);
    const char *group       = ivl_design_flag(des, GROUP_OPTION);

    assert(options != NULL);

#ifdef _VERITOOL_DEBUG_
    printf("application: %s\n", application);
    printf("clock_input: %s\n", clock_input);
    printf("reset_input: %s\n", reset_input);
    printf("rst_poswork: %s\n", rst_poswork);
    printf("cmode: %s\n",       cmode);
    printf("module_name: %s\n", module_name);
    printf("all_outputs: %s\n", all_outputs);
    printf("testbench_v: %s\n", testbench_v);
    printf("vpi_media_c: %s\n", vpi_media_c);
    printf("vpi_media_h: %s\n", vpi_media_h);
    printf("vpi_systf_c: %s\n", vpi_systf_c);
    printf("vpi_systf_h: %s\n", vpi_systf_h);
    printf("interface_c: %s\n", interface_c);
    printf("interface_h: %s\n", interface_h);
    printf("destination: %s\n", destination);
    printf("group: %s\n",       group);
#endif // _VERITOOL_DEBUG_

    if(vt_defined_option(all_outputs))
        { options->outputs = ALL_OUTPUTS; }
    else
    {
        options->outputs =
            (vt_defined_option(testbench_v) ? TESTBENCH_V               : 0x0000) |
            (vt_defined_option(vpi_media_c) ? VPI_MEDIA_C | VPI_MEDIA_H : 0x0000) |
            (vt_defined_option(vpi_systf_c) ? VPI_SYSTF_C | VPI_SYSTF_H : 0x0000) |
            (vt_defined_option(interface_c) ? INTERFACE_C | INTERFACE_H : 0x0000) ;

        if(options->outputs == 0x0000)
            { options->outputs = ALL_OUTPUTS; }
    }

    if(vt_defined_option(rst_poswork)) { options->rst_poswork = 1; }
    if(vt_defined_option(cmode))       { options->cmode = 1; }
    if(vt_defined_option(group))       { options->group = 1; }

    if(vt_defined_and_not_empty_option(application)) { options->application = application; }
    if(vt_defined_and_not_empty_option(clock_input)) { options->clock_input = clock_input; }
    if(vt_defined_and_not_empty_option(reset_input)) { options->reset_input = reset_input; }
    if(vt_defined_and_not_empty_option(module_name)) { options->module_name = module_name; }
    if(vt_defined_and_not_empty_option(all_outputs)) { options->testbench_v = all_outputs; }
    if(vt_defined_and_not_empty_option(testbench_v)) { options->testbench_v = testbench_v; }
    if(vt_defined_and_not_empty_option(vpi_media_c)) { options->vpi_media_c = vpi_media_c; } else { if(options->cmode == 1) { options->vpi_media_c = VPI_MEDIA_C_DEFAULT; } }
    if(vt_defined_and_not_empty_option(vpi_media_h)) { options->vpi_media_h = vpi_media_h; }
    if(vt_defined_and_not_empty_option(vpi_systf_c)) { options->vpi_systf_c = vpi_systf_c; } else { if(options->cmode == 1) { options->vpi_systf_c = VPI_MEDIA_C_DEFAULT; } }
    if(vt_defined_and_not_empty_option(vpi_systf_h)) { options->vpi_systf_h = vpi_systf_h; }
    if(vt_defined_and_not_empty_option(interface_c)) { options->interface_c = interface_c; } else { if(options->cmode == 1) { options->interface_c = INTERFACE_C_DEFAULT; } }
    if(vt_defined_and_not_empty_option(interface_h)) { options->interface_h = interface_h; }
    if(vt_defined_and_not_empty_option(destination)) { options->destination = destination; }

    if(!vt_defined_option(vpi_media_h)) { options->vpi_media_h = vt_get_vpi_media_h(options->vpi_media_c); }
    if(!vt_defined_option(vpi_systf_h)) { options->vpi_systf_h = vt_get_vpi_systf_h(options->vpi_systf_c); }
    if(!vt_defined_option(interface_h)) { options->interface_h = vt_get_interface_h(options->interface_c); }

    return 0;
}
